@@ -1,8 +1,8 @@
 /* Copyright © 2005-2007  Roger Leigh <rleigh@debian.org>
  *
- * schroot is free software; you can redistribute it and/or modify it
+ * schroot is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
  * schroot is distributed in the hope that it will be useful, but
@@ -11,17 +11,20 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
- * MA  02111-1307  USA
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
  *
  *********************************************************************/
 
 #include <config.h>
 
+#include "sbuild-chroot.h"
 #include "sbuild-error.h"
 #include "sbuild-util.h"
 
+#include <cerrno>
+#include <cstring>
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -59,8 +62,32 @@ namespace
     return ret;
   }
 
+  typedef std::pair<sbuild::stat::error_code,const char *> emap;
+
+  /**
+   * This is a list of the supported error codes.  It's used to
+   * construct the real error codes map.
+   */
+  emap init_errors[] =
+    {
+      emap(sbuild::stat::FILE, N_("Failed to stat file")),
+      emap(sbuild::stat::FD,   N_("Failed to stat file descriptor"))
+    };
+
+  bool chroot_alphasort (sbuild::chroot::ptr const& c1,
+			 sbuild::chroot::ptr const& c2)
+  {
+    return c1->get_name() < c2->get_name();
+  }
+
 }
 
+template<>
+error<sbuild::stat::error_code>::map_type
+error<sbuild::stat::error_code>::error_strings
+(init_errors,
+ init_errors + (sizeof(init_errors) / sizeof(init_errors[0])));
+
 std::string
 sbuild::basename (std::string name,
 		  char        separator)
@@ -291,13 +318,15 @@ sbuild::find_program_in_path (std::string const& program,
 	}
       absname += realname;
 
-      struct stat statbuf;
-      if (stat(absname.c_str(), &statbuf) == 0)
+      try
 	{
-	  if (S_ISREG(statbuf.st_mode) &&
+	  if (stat(absname).is_regular() &&
 	      access (absname.c_str(), X_OK) == 0)
 	    return realname;
 	}
+      catch (std::runtime_error const& e)
+	{
+	}
     }
 
   return "";
@@ -346,3 +375,38 @@ sbuild::exec (std::string const& file,
 
   return status;
 }
+
+sbuild::stat::stat (std::string const& file):
+  file(file),
+  fd(0),
+  errorno(0),
+  status()
+{
+  if (::stat(file.c_str(), &this->status) < 0)
+    this->errorno = errno;
+}
+
+sbuild::stat::stat (std::string const& file,
+		    int                fd):
+  file(file),
+  fd(fd),
+  errorno(0),
+  status()
+{
+  if (::fstat(fd, &this->status) < 0)
+    this->errorno = errno;
+}
+
+sbuild::stat::stat (int fd):
+  file(),
+  fd(fd),
+  errorno(0),
+  status()
+{
+  if (::fstat(fd, &this->status) < 0)
+    this->errorno = errno;
+}
+
+sbuild::stat::~stat ()
+{
+}
